@@ -2,6 +2,7 @@
 
 import json
 import os
+import socket
 from pathlib import Path
 from time import time
 
@@ -16,6 +17,7 @@ class ConfigReaderAndWriter:
     GEOSPOCK_FILE = GEOSPOCK_DIR.joinpath("credentials")
     CONFIG_FILE = GEOSPOCK_DIR.joinpath("config.json")
     MESSAGES = json.loads(resource_string('geospock_cli', "messages.json").decode("utf-8"))
+    KEY = socket.gethostname()
 
     def __init__(self, profile: str, show_browser: bool):
         self.profile = profile
@@ -23,9 +25,26 @@ class ConfigReaderAndWriter:
         self.config = None
         self.credentials = None
 
-    def write_to_config(self, client_id: str, audience: str, auth0url: str, request_address: str) -> None:
-        if not os.path.exists(self.GEOSPOCK_DIR):
-            os.mkdir(self.GEOSPOCK_DIR)
+    def encode(self, string):
+        encoded_chars = []
+        for i in range(len(string)):
+            key_c = self.KEY[i % len(self.KEY)]
+            encoded_c = chr(ord(string[i]) + ord(key_c) % 256)
+            encoded_chars.append(encoded_c)
+        encoded_string = "".join(encoded_chars)
+        return encoded_string
+
+    def decode(self, string):
+        decoded_chars = []
+        for i in range(len(string)):
+            key_c = self.KEY[i % len(self.KEY)]
+            decoded_c = chr(ord(string[i]) - ord(key_c) % 256)
+            decoded_chars.append(decoded_c)
+        decoded_string = "".join(decoded_chars)
+        return decoded_string
+
+    def write_login(self, username: str, password: str, request_address: str):
+        Path(self.GEOSPOCK_DIR).mkdir(parents=True, exist_ok=True)
 
         if os.path.exists(self.CONFIG_FILE) and os.path.getsize(self.CONFIG_FILE) > 0:
             with open(self.CONFIG_FILE) as json_file:
@@ -33,7 +52,39 @@ class ConfigReaderAndWriter:
                     current_config = json.load(json_file)
                 except json.JSONDecodeError:
                     raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidConfig"])
-                if self.profile in current_config:
+        else:
+            current_config = dict()
+        try:
+            current_config[self.profile] = dict(l=self.encode(username), p=self.encode(password),
+                                                request_address=request_address)
+        except Exception:
+            raise CLIError("Could not generate configuration entry from provided username and password.")
+        with open(self.CONFIG_FILE, "w") as config_file_write:
+            config_file_write.write(json.dumps(current_config, indent=4))
+
+    def write_logout(self):
+        if os.path.exists(self.CONFIG_FILE) and os.path.getsize(self.CONFIG_FILE) > 0:
+            with open(self.CONFIG_FILE) as json_file:
+                try:
+                    current_config = json.load(json_file)
+                    current_config.pop(self.profile, None)
+                except json.JSONDecodeError:
+                    raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidConfig"])
+        else:
+            current_config = dict()
+        with open(self.CONFIG_FILE, "w") as config_file_write:
+            config_file_write.write(json.dumps(current_config, indent=4))
+
+    def write_to_config(self, client_id: str, audience: str, auth0url: str, request_address: str) -> None:
+        Path(self.GEOSPOCK_DIR).mkdir(parents=True, exist_ok=True)
+
+        if os.path.exists(self.CONFIG_FILE) and os.path.getsize(self.CONFIG_FILE) > 0:
+            with open(self.CONFIG_FILE) as json_file:
+                try:
+                    current_config = json.load(json_file)
+                except json.JSONDecodeError:
+                    raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidConfig"])
+                if self.profile in current_config and "client_id" in current_config[self.profile]:
                     if current_config[self.profile]["client_id"] == client_id and \
                             current_config[self.profile]["audience"] == audience and \
                             current_config[self.profile]["auth0_url"] == auth0url and \
