@@ -3,6 +3,8 @@
 import click
 import json
 import boto3
+import base64
+import os
 
 from geospock_cli.auth import Authorizer
 from geospock_cli.exceptions import CLIError
@@ -13,27 +15,48 @@ from typing import Tuple
 from urllib.parse import urlparse
 
 
-def get_header(config_reader: ConfigReaderAndWriter) -> Tuple[dict, str]:
+def get_header(config_reader: ConfigReaderAndWriter, user: str, password: str, request_address: str) -> Tuple[dict, str]:
     config_all = config_reader.get_config()
     if config_reader.profile not in config_all and config_reader.profile != "default":
         raise CLIError(ConfigReaderAndWriter.MESSAGES["profileError"].format(config_reader.profile))
-    elif config_reader.profile not in config_all:
+    elif config_reader.profile not in config_all and not request_address:
         raise CLIError(ConfigReaderAndWriter.MESSAGES["noInit"].format(config_reader.profile))
-    request_address = config_reader.config["request_address"]
-    all_credentials = config_reader.get_all_credentials_from_file()
-    auth = Authorizer(config_reader)
-    if all_credentials is None or config_reader.profile not in all_credentials:
-        credentials = auth.init_credentials()
-    else:
-        credentials = auth.renew_credentials()
+    if not request_address:
+        request_address = config_reader.config["request_address"]
+    if "auth0_url" in config_all[config_reader.profile]:
+        all_credentials = config_reader.get_all_credentials_from_file()
+        auth = Authorizer(config_reader)
+        if all_credentials is None or config_reader.profile not in all_credentials:
+            credentials = auth.init_credentials()
+        else:
+            credentials = auth.renew_credentials()
 
-    if credentials:
-        headers = {"content-type": "application/json", "authorization": "Bearer " + credentials["access_token"]}
-        return headers, request_address
+        if credentials:
+            headers = {"content-type": "application/json", "authorization": "Bearer " + credentials["access_token"]}
+            return headers, request_address
+        else:
+            raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidCredentials"])
+    elif user is not None:
+        return get_basic_auth_header(user, password, request_address)
+    elif os.environ.get('GEOSPOCK_USER') is not None:
+        return get_basic_auth_header(os.environ.get('GEOSPOCK_USER'),
+                                     os.environ.get('GEOSPOCK_PASSWORD'),
+                                     request_address)
+    elif "l" in config_all[config_reader.profile] and "p" in config_all[config_reader.profile]:
+        return get_basic_auth_header(config_reader.decode(config_all[config_reader.profile]["l"]),
+                                     config_reader.decode(config_all[config_reader.profile]["p"]),
+                                     request_address)
     else:
         raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidCredentials"])
 
 
+def get_basic_auth_header(username: str, password: str, request_address: str) -> Tuple[dict, str]:
+    basic_unencoded = username + ":" + password
+    basic_encoded = base64.b64encode(basic_unencoded.encode("utf-8"))
+    headers = {"content-type": "application/json", "authorization": "Basic %s" % basic_encoded}
+    return headers, request_address
+
+
 def display_result(result: dict, output_object: str, is_stringified_json: bool) -> None:
     try:
         if output_object and is_stringified_json:
@@ -54,20 +77,20 @@ def display_result(result: dict, output_object: str, is_stringified_json: bool)
         raise CLIError(ConfigReaderAndWriter.MESSAGES["graphQLError"] + str(result))
 
 
-def clean_null_items(input):
-    if isinstance(input, dict):
+def clean_null_items(input_value):
+    if isinstance(input_value, dict):
         clean = dict()
-        for key, value in input.items():
+        for key, value in input_value.items():
             clean_value = clean_null_items(value)
             if value is not None:
                 clean[key] = clean_value
         return clean
-    elif isinstance(input, list):
+    elif isinstance(input_value, list):
         new_list = []
-        for item in input:
+        for item in input_value:
             new_list.append(clean_null_items(item))
         return new_list
-    return input
+    return input_value
 
 
 def display_result_without_nulls(result: dict, output_object: str) -> None:
@@ -89,19 +112,23 @@ def init(config_reader, clientid, audience, auth0url, request_address) -> None:
                                                     if config_reader.profile != "default" else ""), fg="green")
 
 
-@click.command(help=ConfigReaderAndWriter.MESSAGES["helpRun"], context_settings=dict(ignore_unknown_options=True, allow_extra_args=True))
+@click.command(help=ConfigReaderAndWriter.MESSAGES["helpRun"], context_settings=dict(ignore_unknown_options=True,
+                                                                                     allow_extra_args=True))
 @click.pass_context
 @click.argument('command')
 @click.option('--profile', default="default", help=ConfigReaderAndWriter.MESSAGES["profileText"])
+@click.option('--user', default=None)
+@click.option('--password', default=None)
+@click.option('--request-address', default=None)
 @click.option('--debug', is_flag=True, help=ConfigReaderAndWriter.MESSAGES["debugText"])
 @click.option('--no-browser', is_flag=True, help=ConfigReaderAndWriter.MESSAGES["noBrowserText"])
-def run(ctx, command, profile, debug, no_browser) -> None:
+def run(ctx, command, profile, debug, no_browser, user, password, request_address) -> None:
     try:
         config_reader = ConfigReaderAndWriter(profile, not no_browser)
         if command == "help" and len(ctx.args) == 0:
-            help_all(config_reader, debug)
+            help_all(config_reader, user, password, request_address, debug)
         elif command == "help":
-            help_command(config_reader, ctx.args[0], debug)
+            help_command(config_reader, ctx.args[0], user, password, request_address, debug)
         elif command == "profile-list":
             profile_list(config_reader)
 
@@ -114,15 +141,32 @@ def run(ctx, command, profile, debug, no_browser) -> None:
             if command == "init":
                 # Saving info to config.json
                 try:
-                    init(config_reader, args["clientid"], args["audience"], args["auth0url"], args["request-address"])
+                    init(config_reader, args["clientid"], args["audience"], args["auth0url"], request_address)
                 except KeyError:
                     raise CLIError(ConfigReaderAndWriter.MESSAGES["helpInit"])
             elif command == "get-credentials":
                 # Add to credentials file
-                if get_header(config_reader):
+                if get_header(config_reader, user, password, request_address):
                     click.secho("Credentials saved", fg="green")
+            elif command == "login":
+                if request_address is None or user is None or password is None:
+                    raise CLIError(ConfigReaderAndWriter.MESSAGES["helpLogin"])
+                try:
+                    config_reader.write_login(user, password, request_address)
+                    logout_command = "geospock logout{}".format(
+                        " --profile " + config_reader.profile if config_reader.profile != "default" else "")
+                    click.secho("Login details saved. Use '{}' to remove these details.".format(logout_command),
+                                fg="green")
+                except KeyError:
+                    raise CLIError(ConfigReaderAndWriter.MESSAGES["helpLogin"])
+            elif command == "logout":
+                try:
+                    config_reader.write_logout()
+                    click.secho("Login details removed.", fg="green")
+                except KeyError:
+                    raise CLIError(ConfigReaderAndWriter.MESSAGES["helpLogin"])
             else:
-                run_command(command, config_reader, args, debug)
+                run_command(command, config_reader, args, user, password, request_address, debug)
     except CLIError as e:
         click.secho(str(e.message), fg="red")
         exit(e.exit_code)
@@ -149,9 +193,10 @@ def load_file(address):
         raise CLIError("Error trying to read JSON in input file: {}".format(address))
 
 
-def run_command(command: str, config_reader: ConfigReaderAndWriter, args: dict, debug: bool) -> None:
+def run_command(command: str, config_reader: ConfigReaderAndWriter, args: dict,
+                user: str, password: str, request_address: str, debug: bool) -> None:
     # Where we have an argument ending in the following, read from a file rather than taking the direct string
-    response_json = get_command_from_server(config_reader, command, debug)
+    response_json = get_command_from_server(config_reader, command, user, password, request_address, debug)
     if response_json is None:
         click.echo("Could not get command {} from service".format(command))
     if "error" in response_json:
@@ -163,7 +208,8 @@ def run_command(command: str, config_reader: ConfigReaderAndWriter, args: dict,
         new_args, arg_missing = process_parameters(parameters, args, command)
         if not arg_missing:
             request = {"operationName": internal_name, "query": graphql, "variables": new_args}
-            response_data_json = get_from_server(config_reader, request, internal_name, debug)
+            response_data_json = get_from_server(config_reader, request, internal_name,
+                                                 user, password, request_address, debug)
             if response_json.get("stripNulls", False):
                 display_result_without_nulls(response_data_json, internal_name)
             else:
@@ -217,8 +263,8 @@ def check_if_valid_int(parameter_name, value_to_check):
         raise CLIError(f"Value for --{parameter_name} should be an integer - found value '{value_to_check}'")
 
 
-def help_all(config_reader: ConfigReaderAndWriter, debug: bool) -> None:
-    commands_json = get_commands_from_server(config_reader, debug)
+def help_all(config_reader: ConfigReaderAndWriter, user: str, password: str, request_address: str, debug: bool) -> None:
+    commands_json = get_commands_from_server(config_reader, user, password, request_address, debug)
     max_len = max([len(key) for key in commands_json])
     click.echo(ConfigReaderAndWriter.MESSAGES["helpAllCommands"])
     for command in commands_json:
@@ -238,8 +284,9 @@ def help_all(config_reader: ConfigReaderAndWriter, debug: bool) -> None:
         click.echo()
 
 
-def help_command(config_reader: ConfigReaderAndWriter, command: str, debug: bool) -> None:
-    command_json = get_command_from_server(config_reader, command, debug)
+def help_command(config_reader: ConfigReaderAndWriter, command: str,
+                 user: str, password: str, request_address, debug: bool) -> None:
+    command_json = get_command_from_server(config_reader, command, user, password, request_address, debug)
     if command_json and "error" in command_json:
         raise CLIError(command_json["error"])
     elif command_json:
@@ -276,8 +323,9 @@ def profile_list(config_reader: ConfigReaderAndWriter) -> None:
         click.echo(f"--profile {profile}")
 
 
-def get_from_server(config_reader: ConfigReaderAndWriter, request: dict, object_name: str, debug) -> dict:
-    header, request_address = get_header(config_reader)
+def get_from_server(config_reader: ConfigReaderAndWriter, request: dict, object_name: str,
+                    user: str, password: str, request_address: str, debug) -> dict:
+    header, request_address = get_header(config_reader, user, password, request_address)
 
     if header:
         try:
@@ -310,16 +358,18 @@ def get_from_server(config_reader: ConfigReaderAndWriter, request: dict, object_
         raise CLIError(ConfigReaderAndWriter.MESSAGES["noHeader"])
 
 
-def get_commands_from_server(config_reader: ConfigReaderAndWriter, debug: bool) -> dict:
+def get_commands_from_server(config_reader: ConfigReaderAndWriter,
+                             user: str, password: str, request_address: str, debug: bool) -> dict:
     graphql = resource_string('geospock_cli', "graphql_templates/getCommands.graphql").decode("utf-8")
     request = {"operationName": "getCliCommands", "query": graphql, "variables": {}}
-    return get_from_server(config_reader, request, "getCliCommands", debug)
+    return get_from_server(config_reader, request, "getCliCommands", user, password, request_address, debug)
 
 
-def get_command_from_server(config_reader: ConfigReaderAndWriter, command: str, debug: bool) -> dict:
+def get_command_from_server(config_reader: ConfigReaderAndWriter, command: str,
+                            user: str, password: str, request_address: str, debug: bool) -> dict:
     graphql = resource_string('geospock_cli', "graphql_templates/getCommand.graphql").decode("utf-8")
     request = {"operationName": "getCliCommand", "query": graphql, "variables": {"commandName": command}}
-    return get_from_server(config_reader, request, "getCliCommand", debug)
+    return get_from_server(config_reader, request, "getCliCommand", user, password, request_address, debug)
 
 
 if __name__ == '__main__':
