@@ -21,8 +21,14 @@ class MessageSender:
         else:
             conn = http.client.HTTPConnection(endpoint)
         conn.request(method, path, body, headers)
-        response = conn.getresponse().read().decode()
+        response = conn.getresponse()
 
-        if response == "Authentication Error":
-            raise CLIError(response)
-        return response
+        if response.status >= 500:
+            raise CLIError("Could not communicate with the GeoSpock management service - error code {}"
+                           .format(response.status))
+
+        response_body = response.read().decode()
+
+        if response_body == "Authentication Error":
+            raise CLIError(response_body)
+        return response_body
